;Rutinas de acceso a eeprom (externa / interna)

;**************************************************************************************************
;* RUTINAS GESTION BUS I2C                                                                        *
;**************************************************************************************************
i2c_test
		call    i2c_start
		movlw   0xA0        
		goto    i2c_write_byte
;--------------------------------------------------------------------
i2c_controlW 	MACRO
		LOCAL 	i2c_controlWLoop
i2c_controlWLoop
		call    i2c_start
		movlw   0xA0        
		call    i2c_write_byte
		btfsc   STATUS,C
		goto    i2c_controlWLoop
		ENDM
;--------------------------------------------------------------------
i2c_controlR	MACRO
		LOCAL	i2c_controlRLoop
i2c_controlRLoop
		call    i2c_start
		movlw   0xA1
		call    i2c_write_byte
		btfsc   STATUS,C
		goto    i2c_controlRLoop
		ENDM
;--------------------------------------------------------------------
;Send a byte stored in W to external EEPROM
i2c_write_byte 
		movwf   EEDATA2
		movlw   B'11001111'
		tris    PORTB                ; Configura SDA como salida
		movlw	0x08
		movwf	TEMP+5
i2c_write_byte_loop
		rlf	EEDATA2,f
		call	i2c_write_bit_x
		decfsz	TEMP+5,f
		goto	i2c_write_byte_loop
		goto    i2c_read_bit

;--------------------------------------------------------------------
;Read a byte from external EEPROM into EEDATA2
i2c_read_byte    
		movlw   B'11011111'		;load value
		tris    PORTB			;Configura SDA como entrada
		movlw	0x08
		movwf	TEMP+5
i2c_read_byte_loop
		call    i2c_read_bit_x
		rlf     EEDATA2,f
		decfsz	TEMP+5,f
		goto	i2c_read_byte_loop
		return
;--------------------------------------------------------------------
;Init data transfer from/to external EEPROM
i2c_start   
		bsf     PORTB,4		;SDA -> High
		movlw   B'11001111'	;Load value
		tris    PORTB		;Define SDA and SCLK output
		bsf     PORTB,5		;Clock High
;		nop
		bcf     PORTB,4		;Data start
;		goto	clock_low
		bcf	PORTB,5		;probando comunicacin i2c
		return
;--------------------------------------------------------------------
;Stop data transfer from/to external EEPROM
i2c_stop
		bcf     PORTB,4		;Data start
		movlw   B'11001111'	;Load value
		tris    PORTB		;SDA como salida
;		nop
		bsf     PORTB,5	;Clock High
;		nop
		bsf     PORTB,4	;Data stop
nclock_low
;		nop
clock_low	bcf     PORTB,5		;Clock Low
		return

;--------------------------------------------------------------------
i2c_write_bit  
		movlw	B'11001111'
		tris	PORTB           ;SDA como salida
;Actual bit write to RB4 (external EEPROM)
;Bit to be written is from carry
i2c_write_bit_x  
		btfss	STATUS,C		;Skip if carry set
			bcf	PORTB,4     ;SDA -> Low
		btfsc	STATUS,C
			bsf	PORTB,4     ;SDA -> High
		bsf	PORTB,5     ;Clock High
		goto	nclock_low

;--------------------------------------------------------------------
i2c_read_bit    
		movlw   B'11011111'
		tris    PORTB       ;SDA as input
;Actual bit read from RB4 (external EEPROM)
;Bit is read into carry
i2c_read_bit_x   
		bsf     PORTB,5     ;SCL -> High
;		nop
		bsf     STATUS,C
		btfss   PORTB,4     ;Test SDA
			bcf     STATUS,C
;		goto	clock_low
                bcf     PORTB,5     ;SCL -> Low probando comunicacin i2c
                return

;**************************************************************************************************
;* LEER UN BYTE DE LA EEPROM DEL PIC                                                              *
;**************************************************************************************************
;* W = Direccin y Byte leido al salir                                                            *
;**************************************************************************************************
Rd_eeprom   
                bsf     STATUS,RP1      ;Select Bank 2
                movwf   EEADR
                bsf     STATUS,RP0      ;Select Bank 3
                bcf     EECON1,EEPGD
                bsf     EECON1,RD
                bcf     STATUS,RP0      ;Select Bank 2
                movf    EEDATA,W
                bcf     STATUS,RP1      ;Select Bank 0
                return
;**************************************************************************************************
;* ESCRIBE UN BYTE A LA EEPROM DEL PIC                                                            *
;**************************************************************************************************
;* EEADR2 = Direccin  W=Dato                                                                     *
;**************************************************************************************************
Wr_eeprom   
                bcf     PIR2,EEIF
                bsf     STATUS,RP1      ;Select Bank 2
                movwf   EEDATA
                bcf     STATUS,RP1      ;Select Bank 0
                movf    EEADR2,W
                bsf     STATUS,RP1      ;Select Bank 2
                movwf   EEADR
                bsf     STATUS,RP0      ;Select Bank 3
                bcf     EECON1,EEPGD
                bsf     EECON1,WREN
                movlw   0x55
                movwf   EECON2
                movlw   0xAA
                movwf   EECON2
                bsf     EECON1,WR
                bcf     STATUS,RP0      
                bcf     STATUS,RP1      ;Select Bank 0

		btfss   PIR2,EEIF
		goto    $-1

		bsf     STATUS,RP0
		bsf     STATUS,RP1      ;Select Bank 3

		bcf     EECON1,WREN
		bcf     STATUS,RP0
		bcf     STATUS,RP1      ;Select Bank 0
		return

;**************************************************************************************************
;* READ MEMORY FLASH/EXTERNAL                                                                     *
;**************************************************************************************************
;* EEADRH2:W = Address of Memory                                                                  *
;* W = Data Read                                                                                  *
;**************************************************************************************************
Rd_Memory       
		btfss	CONTROL,2
			goto	Rd_Flash
		goto	Rd_ExtMem
Rd_Memory_Next
		btfss	CONTROL,2
			goto	Rd_Flash_Next
		goto	Rd_ExtMem_Next
;**************************************************************************************************
;* WRITE MEMORY FLASH/EXTERNAL                                                                    *
;**************************************************************************************************
;* EEADRH2:EEADR2 = Address of Memory                                                             *
;* W = Data Written                                                                               *
;**************************************************************************************************
Wr_Memory       
		btfss	CONTROL,2
			goto	Wr_Flash
		goto	Wr_ExtMem


;**************************************************************************************************
;* READ EXTERNAL EEPROM MEMORY                                                                    *
;**************************************************************************************************
;* EEADRH2:W = Address of Memory                                                                  *
;* W = Data Read                                                                                  *
;**************************************************************************************************
Rd_ExtMem
		movwf	EEADR2
Rd_ExtMem2              
		i2c_controlW
		movfw	EEADRH2
		call	i2c_write_byte
		movfw	EEADR2
		call	i2c_write_byte
Rd_ExtMem_Next
		i2c_controlR
		call    i2c_read_byte
		call    i2c_stop
		movfw	EEDATA2
		return
;**************************************************************************************************
;* WRITE EXTERNAL EEPROM MEMORY                                                                   *
;**************************************************************************************************
;* EEADRH2:EEADR2 = Address of Flash Memory                                                       *
;* W = Data Written                                                                               *
;**************************************************************************************************
Wr_ExtMem
                bsf     STATUS,RP1      ;Select Bank 2
                movwf   EEDATA
                bcf     STATUS,RP1      ;Select Bank 0
Wr_ExtMem2              
                i2c_controlW
                movf    EEADRH2,W
                call    i2c_write_byte
                movf    EEADR2,W
                call    i2c_write_byte
                bsf     STATUS,RP1      ;Select Bank 2
                movf    EEDATA,W
                bcf     STATUS,RP1      ;Select Bank 0
                call    i2c_write_byte
                goto    i2c_stop

;**************************************************************************************************
;* READ FLASH MEMORY                                                                              *
;**************************************************************************************************
;* EEADRH2:w = Address of Memory                                                                  *
;* W = Data Read                                                                                  *
;**************************************************************************************************
Rd_Flash
                bsf     STATUS,RP1      ;Select Bank 2
                movwf   EEADR
                bcf     STATUS,RP1      ;Select Bank 0
		movlw	HIGH(BASE_DATOS) ;Mapearlo en el segundo 4K bloque
                addwf   EEADRH2,W
                bsf     STATUS,RP1      ;Select Bank 2
                movwf   EEADRH
		goto	Rd_Flash_Next

;******************************************************
;* READ PROGRAM FLASH MEMORY
;******************************************************
;* EEADRH2:EEADR2 = Address of Flash Memory
;* W = Data Read
;******************************************************
Rd_PFlash
                movf    EEADR2,W
                bsf     STATUS,RP1      ;Select Bank 2
                movwf   EEADR
                bcf     STATUS,RP1      ;Select Bank 0
                movf    EEADRH2,W
                bsf     STATUS,RP1      ;Select Bank 2
                movwf   EEADRH
Rd_Flash_Next
                bsf     STATUS,RP1      
                bsf     STATUS,RP0      ;Select Bank 3
                bsf     EECON1,EEPGD
                bsf     EECON1,RD
                nop                     ;Any instructions here are ignored as program
                nop                     ;memory is read in third cycle after BSF EECON1,RD
                bcf     STATUS,RP0      ;Select Bank 2
		movf	EEDATA,W
                incf    EEADR,F
                btfsc   STATUS,Z
                incf    EEADRH,F
		bcf     STATUS,RP1      ;Select Bank 0              
		return

;**************************************************************************************************
;* WRITE FLASH MEMORY                                                                             *
;**************************************************************************************************
;* EEADRH2:EEADR2 = Address of Flash Memory                                                       *
;* W = Data Written                                                                               *
;**************************************************************************************************
Wr_Flash
		bsf     STATUS,RP1      ;Select Bank 2
		movwf	EEDATA
		clrf	EEDATH
		bcf     STATUS,RP1      ;Select Bank 0
		movlw	HIGH(BASE_DATOS)
		addwf	EEADRH2,W
		goto	Wr_PFlash1

;******************************************************
;* WRITE PROGRAM FLASH MEMORY
;******************************************************
;* EEADRH2:EEADR2 = Address of Flash Memory
;* EEDATH:EEDATA = Data to Write
;******************************************************
Wr_PFlash
                movf    EEADRH2,W
Wr_PFlash1      bsf     STATUS,RP1      ;Select Bank 2
                movwf   EEADRH
                bcf     STATUS,RP1      ;Select Bank 0
                movf    EEADR2,W
                bsf     STATUS,RP1      ;Select Bank 2
                movwf   EEADR
                bsf     STATUS,RP0      ;Select Bank 3
                bsf     EECON1,EEPGD
                bsf     EECON1,WREN
                movlw   0x55
                movwf   EECON2
                movlw   0xAA
                movwf   EECON2
                bsf     EECON1,WR
                bcf     STATUS,RP0      
                bcf     STATUS,RP1      ;Select Bank 0
                btfss   PIR2,EEIF
	IFNDEF	SIMMODE
                goto    $-1
	ENDIF
                bcf     PIR2,EEIF
                bsf     STATUS,RP0
                bsf     STATUS,RP1      ;Select Bank 3
                bcf     EECON1,WREN
                bcf     STATUS,RP0      ;Select Bank 2
                bcf     STATUS,RP1      ;Select Bank 0
                return
